from __future__ import annotations

from tach.parsing.config import (
    dump_project_config_to_toml,
    extend_and_validate,
    parse_project_config,
)

__all__ = ["parse_project_config", "dump_project_config_to_toml", "extend_and_validate"]
